import { EventTrigger } from './utility';
declare global {
    interface SymbolConstructor {
        readonly observable: unique symbol;
    }
}
export interface Observer<T = any> {
    next(value: T): void;
    error(reason: string | Error): void;
    complete(): void;
}
export interface Subscription {
    unsubscribe(): void;
    readonly closed: boolean;
}
export interface Subscribable<T = any> {
    [Symbol.observable](): Subscribable<T>;
    subscribe(onNext: Observer<T>['next'], onError?: Observer<T>['error'], onComplete?: Observer<T>['complete']): Subscription;
}
export declare type SubscriberFunction<T = any> = (observer: Observer<T>) => (() => void) | void;
export declare class Observable<T = any> implements Subscribable {
    private subscriber;
    constructor(subscriber: SubscriberFunction<T>);
    [Symbol.observable](): this;
    [Symbol.asyncIterator](): AsyncGenerator<T, void, unknown>;
    static of<T = any>(...items: T[]): Observable<T>;
    toPromise(): Promise<any>;
    subscribe(onNext: Observer<T>['next'], onError?: Observer<T>['error'], onComplete?: Observer<T>['complete']): {
        unsubscribe(): void;
        readonly closed: boolean;
    };
    static from<T = any>(observable: Subscribable<T>): Observable<T>;
    static fromEvent<T = any>(target: EventTrigger, name: string): Observable<T>;
}
